@@ -25,22 +25,22 @@
 /// Provide a panic-like macro that can be used in the framework to detect a test failure and save
 /// that status with a message for later screen printing.
 ///
-/// std unit tests rely heavily on unwinding. Each unit test is run inside a catch_unwind block and
-/// if the unit test panics then the panic is caught and the test is marked as failed
-/// (or as passed if the unit test was marked with `#[should_panic]`).
+/// std unit tests rely heavily on unwinding. Each unit test is run inside a catch_unwind block.
+/// If the unit test panics then the panic is caught and the test is marked as 'failed'
+/// (or as 'passed' if the unit test was marked with `#[should_panic]`).
 ///
-/// The framework attempt to emulate this behavior by by-passing the panic macro to mark the test
+/// The framework attempts to emulate this behavior by by-passing the panic macro to mark the test
 /// failed and then early return instead of unwind. Of course, this emulation doesn't work on panic!
 /// originates from outside the crate under test, because panic is not overridden in that scope.
-/// This macro provide a way to differentiate between true panic and test failure one.
+/// This macro provides a way to differentiate between true panic and test failure one.
 ///
-/// But this emulation has some limitation. For instance, it can only cause one panicking function to
+/// This emulation has some limitations. For instance, it can only causes one panicking function to
 /// return. To remediate on that, any subsequent panic! call will directly return causing some panic
 /// to be invisible until they get the chance to be the first one. To provide a bit of insight, the
-/// macro also set a `panicking` flag to let you know that an other panic was thrown after the first
-/// one (but without any messages or context). If a panicking panic is triggered again, the framework
-/// will considered either the tests ill written or the failure too fatal and cause a real panic to
-/// cause an abort!
+/// macro also sets a `panicking` flag to let you know that another panic was thrown after the first
+/// one (but without any messages or context).
+/// If a panicking panic is triggered again, the framework will consider either the tests are ill written
+/// or the failure is too fatal. In this case real panic will cause an abort!
 #[macro_export]
 macro_rules! substance_panic {
     () => (
@@ -51,15 +51,13 @@ macro_rules! substance_panic {
     });
     ($fmt:expr, $($arg:tt)*) => ({
         #[allow(improper_ctypes)]
-        extern "Rust" {
-            static mut STATE: substance_framework::State<'static>;
-        }
-        unsafe {
-            if STATE.panic {
+        {
+            let mut state = substance_framework::STATE.write();
+            if state.panic {
                 substance_framework::crash(file!(), line!(), column!(), Some(&format_args!("{}", "panic is panicking again")));
             } else {
-                STATE.panic = true;
-                STATE.msg = format_args!($fmt, $($arg)*).as_str();
+                state.panic = true;
+                state.msg = format_args!($fmt, $($arg)*).as_str();
                 return
             }
         }
@@ -68,8 +66,8 @@ macro_rules! substance_panic {
 
 //////////////////////////////////////////////
 
-/// Bypass the default panic macro and redirect all its call to our custom implementation. This is
-/// necessary to prevent unwinding and let us detect test failure with some assertions.
+/// Bypass the default panic macro and redirect all its calls to our custom implementation.
+/// This is necessary to prevent unwinding and let us detect test failure with some assertions.
 ///
 /// cfr. [substance_panic!]
 #[macro_export]
@@ -81,13 +79,10 @@ macro_rules! panic {
 
 //////////////////////////////////////////////
 
-//TODO: Avoid duplication
-
-//////////////////////////////////////////////
-
 /// Passes if Condition is true.
 ///
-/// Evaluates the condition and passes if it is true. Otherwise the test is marked as failure.
+/// Evaluates the condition and passes if it is true.
+/// Otherwise, the test is marked as failure.
 /// The optional string is printed on failure.
 ///
 /// # Examples
@@ -128,9 +123,10 @@ macro_rules! cx_assert {
 
 //////////////////////////////////////////////
 
-/// Passes if both argument are equals.
+/// Passes if both arguments are equals.
 ///
-/// Passes if the left argument is equal to the right argument. Otherwise the test is marked as failure.
+/// Passes if the left argument is equal to the right argument.
+/// Otherwise, the test is marked as failure.
 ///
 /// # Examples
 ///
@@ -167,9 +163,10 @@ macro_rules! cx_assert_eq {
 
 //////////////////////////////////////////////
 
-/// Passes if both argument are not equals.
+/// Passes if both arguments are not equal.
 ///
-/// Passes if the left argument is not equal to the right argument. Otherwise the test is marked as failure.
+/// Passes if the left argument is not equal to the right argument.
+/// Otherwise, the test is marked as failure.
 ///
 /// # Examples
 ///
@@ -206,9 +203,10 @@ macro_rules! cx_assert_ne {
 
 //////////////////////////////////////////////
 
-/// Passes if the left argument is smaller than the right one.
+/// Passes if the left argument is strictly smaller than the right one.
 ///
-/// Passes if the left argument is smaller than the right argument. Otherwise the test is marked as failure.
+/// Passes if the left argument is strictly smaller than the right argument.
+/// Otherwise, the test is marked as failure.
 ///
 /// # Examples
 ///
@@ -245,9 +243,10 @@ macro_rules! cx_assert_lt {
 
 //////////////////////////////////////////////
 
-/// Passes if the left argument is smaller or equals than the right one.
+/// Passes if the left argument is strictly smaller than or equal to the right one.
 ///
-/// Passes if the left argument is smaller or equals than the right argument. Otherwise the test is marked as failure.
+/// Passes if the left argument is strictly smaller than or equal to the right argument.
+/// Otherwise, the test is marked as failure.
 ///
 /// # Examples
 ///
@@ -288,9 +287,10 @@ macro_rules! cx_assert_le {
 
 //////////////////////////////////////////////
 
-/// Passes if the left argument is greater than the right one.
+/// Passes if the left argument is strictly greater than the right one.
 ///
-/// Passes if the left argument is greater than the right argument. Otherwise the test is marked as failure.
+/// Passes if the left argument is strictly greater than the right argument.
+/// Otherwise, the test is marked as failure.
 ///
 /// # Examples
 ///
@@ -327,9 +327,10 @@ macro_rules! cx_assert_gt {
 
 //////////////////////////////////////////////
 
-/// Passes if the left argument is greater or equals than the right one.
+/// Passes if the left argument is strictly greater than or equals to the right one.
 ///
-/// Passes if the left argument is greater or equals than the right argument. Otherwise the test is marked as failure.
+/// Passes if the left argument is strictly greater than or equals to the right argument.
+/// Otherwise, the test is marked as failure.
 ///
 /// # Examples
 ///
@@ -367,3 +368,87 @@ macro_rules! cx_assert_ge {
         }
     });
 }
+
+//////////////////////////////////////////////
+
+/// Passes if the argument is an Ok result.
+///
+/// Passes if the argument is an instance of Result::Ok(T).
+/// Otherwise, the test is marked as failure.
+///
+/// # Examples
+///
+/// ```rust
+/// cx_assert_ok(my_function_ok());
+/// ```
+///
+/// ```rust
+/// let empty = Result::Ok();
+/// cx_assert_ok(empty);
+/// ```
+///
+/// ```rust
+/// let err = Result::Err();
+/// cx_assert_ok(err, "this test is a failure");
+/// ```
+#[macro_export]
+macro_rules! cx_assert_ok {
+    ($cond:expr $(,)?) => ({
+        match (&$cond) {
+            cond_result => {
+                if ! *cond_result.is_ok() {
+                    substance_panic!("assertion failed: {} is Ok", &*cond_result);
+                }
+            }
+        }
+    });
+    ($cond:expr, $($arg:tt)+) => ({
+        match (&$cond) {
+            cond_bool => {
+                if ! *cond_result.is_ok() {
+                    substance_panic!("assertion failed: {} is Ok  >> {}", &*cond_result, format_args!($($arg)+));
+                }
+            }
+        }
+    });
+}
+
+//////////////////////////////////////////////
+
+/// Passes if the argument is an existing optional value.
+///
+/// Passes if the argument is an instance of Option::Some(T).
+/// Otherwise, the test is marked as failure.
+///
+/// # Examples
+///
+/// ```rust
+/// let empty = Option::Some();
+/// cx_assert_some(empty);
+/// ```
+///
+/// ```rust
+/// let err = Option::None();
+/// cx_assert_some(err, "this test is a failure");
+/// ```
+#[macro_export]
+macro_rules! cx_assert_some {
+    ($cond:expr $(,)?) => ({
+        match (&$cond) {
+            cond_result => {
+                if ! *cond_result.is_some() {
+                    substance_panic!("assertion failed: {} is Some", &*cond_result);
+                }
+            }
+        }
+    });
+    ($cond:expr, $($arg:tt)+) => ({
+        match (&$cond) {
+            cond_bool => {
+                if ! *cond_result.is_some() {
+                    substance_panic!("assertion failed: {} is Some  >> {}", &*cond_result, format_args!($($arg)+));
+                }
+            }
+        }
+    });
+}
