@@ -23,63 +23,66 @@
  ***********************************************************************************/
 
 pub use core::intrinsics::abort;
-use core::intrinsics::likely;
 pub use core::panic::{Location, PanicInfo};
 
 //////////////////////////////////////////////
 
-use crate::{Conclusion, Outcome, STATE};
+use crate::{Conclusion, Outcome, Output, STATE};
 
 //////////////////////////////////////////////
 
 #[allow(clippy::missing_safety_doc)]
 #[panic_handler]
 #[no_mangle]
-pub unsafe fn cx_panic(info: &PanicInfo) -> ! {
+pub fn cx_panic(info: &PanicInfo) -> ! {
     let location = info.location();
     let message = info.message();
 
-    if likely(STATE.printer.is_some()) {
-        let printer = STATE.printer.as_mut().unwrap();
-        let mut default = Conclusion {
-            has_aborted: true,
-            num_ignored: usize::MAX,
-            num_passed: 0,
-            num_failed: usize::MAX,
-            num_skipped: usize::MAX,
-        };
-        let mut conclusion = match STATE.conclusion.as_mut() {
-            Some(c) => {
-                c.has_aborted = true;
-                c
-            }
-            None => &mut default,
-        };
+    let mut conclusion = Conclusion {
+        has_aborted: true,
+        num_ignored: usize::MAX,
+        num_passed: 0,
+        num_failed: usize::MAX,
+        num_skipped: usize::MAX,
+    };
 
-        if likely(STATE.current.is_some()) {
-            let outcome = Outcome::Aborted {
-                reason: match message {
-                    Some(msg) => msg.as_str(),
-                    None => None,
-                },
-            };
-            conclusion.num_skipped -= 1;
-            STATE.counter += 1;
-            printer.print_single_outcome(&outcome);
-        }
+    unsafe { STATE.force_write_unlock() }
 
-        let outcome = Outcome::Skipped;
-        for index in STATE.counter..STATE.tests.unwrap().len() {
-            printer.print_test(STATE.tests.unwrap()[index].name);
-            printer.print_single_outcome(&outcome);
-        }
+    let state = STATE.read();
+    let printer = state.printer.lock();
 
-        printer.print_summary(conclusion);
+    if state.conclusion.is_some() {
+        conclusion.num_skipped = state.conclusion.as_ref().unwrap().num_skipped;
+        conclusion.num_ignored = state.conclusion.as_ref().unwrap().num_ignored;
+        conclusion.num_passed = state.conclusion.as_ref().unwrap().num_passed;
+        conclusion.num_failed = state.conclusion.as_ref().unwrap().num_failed;
     }
 
+    let mut counter = state.counter;
+
+    if state.current.is_some() {
+        let outcome = Outcome::Aborted {
+            reason: match message {
+                Some(msg) => msg.as_str(),
+                None => None,
+            },
+        };
+        conclusion.num_skipped -= 1;
+        counter += 1;
+        printer.print_single_outcome(&outcome);
+    }
+
+    let outcome = Outcome::Skipped;
+    for index in counter..state.tests.unwrap().len() {
+        printer.print_test(state.tests.unwrap()[index].name);
+        printer.print_single_outcome(&outcome);
+    }
+
+    printer.print_summary(&conclusion);
+
     libc_println!();
 
-    if likely(message.is_some() && location.is_some()) {
+    if message.is_some() && location.is_some() {
         libc_println!(
             "[PANIC] {}:{}:{}",
             location.unwrap().file(),
@@ -97,7 +100,7 @@ pub unsafe fn cx_panic(info: &PanicInfo) -> ! {
             location.unwrap().column()
         );
     } else {
-        libc_println!("[PANIC] A panic occurred but not information available");
+        libc_println!("[PANIC] A panic occurred but no information is available");
         libc_println!("    >>> {:?}", info);
     }
     libc_println!();
@@ -108,13 +111,13 @@ pub unsafe fn cx_panic(info: &PanicInfo) -> ! {
 #[allow(clippy::missing_safety_doc)]
 #[lang = "eh_personality"]
 #[no_mangle]
-pub unsafe fn rust_eh_personality() -> ! {
+pub fn rust_eh_personality() -> ! {
     unreachable!("eh_personality")
 }
 
 #[allow(clippy::missing_safety_doc)]
 #[allow(non_snake_case)]
 #[no_mangle]
-pub unsafe fn _Unwind_Resume() -> ! {
+pub fn _Unwind_Resume() -> ! {
     unreachable!("_Unwind_Resume")
 }
